/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 * Copyright (C) 2000 Greg Haerr <greg@censoft.com>
 * Copyright (C) 2000 Alex Holden <alex@linuxhacker.org>
 */
#include <stdio.h>
#include <stdlib.h>
#define MWINCLUDECOLORS
#include "nano-X.h"
#include "nxdraw.h"
/* Uncomment this if you want debugging output from this file */
//#define DEBUG
#include "nanowm.h"
#include "apps.h"
#include "catagories.h"

/* Where to place the first window on the screen */
#define FIRST_WINDOW_LOCATION 2

/* The distance to leave between windows when deciding where to place */
#define WINDOW_STEP 20

static GR_COORD lastx = FIRST_WINDOW_LOCATION;
static GR_COORD lasty = FIRST_WINDOW_LOCATION;

static void client_wndproc(win *window, GR_EVENT *ep);

/*
 * A new client window has been mapped, so we need to reparent and decorate it.
 * Returns -1 on failure or 0 on success.
 */
int
client_updatemap_new(GR_WINDOW_ID wid)
{
	GR_WINDOW_ID 	pid;
	GR_COORD 	x, y, width, height, xoffset, yoffset;
	GR_SIZE		w, h;
	GR_WM_PROPS 	style;
	GR_WINDOW_INFO 	winfo;
	GR_WM_PROPERTIES props;
	static GR_SCREEN_INFO 	si;

	/* get screen/workspace information*/
	if (!si.rows)
		GrGetScreenInfo(&si);

	/* get client window information*/
	GrGetWindowInfo(wid, &winfo);
	style = winfo.props;

	/* if not redecorating or not child of root window, return*/
	if (winfo.parent != GR_ROOT_WINDOW_ID ||
	    (style & GR_WM_PROPS_NODECORATE))
		return 0;

	/* deal with replacing borders with window decorations*/
	if (winfo.bordersize) {
		/*
		 * For complex reasons, it's easier to unmap,
		 * remove the borders, and then map again,
		 * rather than try to recalculate the window
		 * position in the server w/o borders.  By
		 * the time we get this event, the window has
		 * already been painted with borders...
		 * This currently causes a screen flicker as
		 * the window is painted twice.  The workaround
		 * is to create the window without borders in
		 * the first place.
		 */
		GrUnmapWindow(wid);

		/* remove client borders, if any*/
		props.flags = style | GR_WM_FLAGS_BORDERSIZE;
		props.bordersize = 0;
		GrSetWMProperties(wid, &props);

		/* remap the window without borders, call this routine again*/
		GrMapWindow(wid);
		return 0;
	}
	
	/* if default decoration style asked for, set real draw bits*/
	if ((style & GR_WM_PROPS_APPMASK) == GR_WM_PROPS_APPWINDOW) {
		GR_WM_PROPERTIES pr;

		style = (style & ~GR_WM_PROPS_APPMASK) |
			nxGetDefaultWindowStyle();

		/* A quick hack - if NOAUTORESIZE is enabled */
		/* then clear the NOMOVE flag */
		
		if (style & GR_WM_PROPS_NOAUTORESIZE)
		  style &= ~GR_WM_PROPS_NOMOVE;
		
		pr.flags = GR_WM_FLAGS_PROPS;
		pr.props = style;
		GrSetWMProperties(wid, &pr);
	}

	/* determine container size and client child window offsets*/
	nxCalcNCSize(style, winfo.width, winfo.height, &xoffset, &yoffset,
		&width, &height);

	/* determine x,y window location and width,height size*/
	if ((style & (GR_WM_PROPS_NOAUTORESIZE|GR_WM_PROPS_MAXIMIZE)) == 
	    GR_WM_PROPS_MAXIMIZE) {

		/* force to maximized position*/
		x = 0; y = 0;
		width = si.ws_width;
		height = si.ws_height;
	} else if (style & GR_WM_PROPS_NOAUTOMOVE) {
		x = winfo.x;
		y = winfo.y;
	} else {

		/* We could probably use a more intelligent algorithm here */
		x = lastx + WINDOW_STEP;
		if((x + width) > si.cols)
			x = FIRST_WINDOW_LOCATION;
		lastx = x;
		y = lasty + WINDOW_STEP;
		if((y + height) > si.rows)
			y = FIRST_WINDOW_LOCATION;
		lasty = y;
	}

	/* create container window*/
	pid = GrNewWindow(GR_ROOT_WINDOW_ID, x, y, width, height,
		0, LTGRAY, BLACK);
	add_window(pid, GR_ROOT_WINDOW_ID, wid, container_wndproc);

	/* don't erase background of container window*/
	props.flags = GR_WM_FLAGS_PROPS;
	props.props = style | GR_WM_PROPS_NOBACKGROUND;
	GrSetWMProperties(pid, &props);

	Dprintf("New client window %d container %d\n", wid, pid);

	GrSelectEvents(pid, GR_EVENT_MASK_CHLD_UPDATE | GR_EVENT_MASK_UPDATE
		| GR_EVENT_MASK_BUTTON_UP | GR_EVENT_MASK_BUTTON_DOWN
		| GR_EVENT_MASK_MOUSE_POSITION | GR_EVENT_MASK_EXPOSURE);

	/* reparent client to container window*/
	/* must map before reparent (nano-X bug)*/
	GrMapWindow(pid);
	GrReparentWindow(wid, pid, xoffset, yoffset);

	/* resize client window to container*/
	nxCalcClientSize(style, width, height, NULL, NULL, &w, &h);
	if (w != winfo.width || h != winfo.height)
		GrResizeWindow(wid, w, h);

	/* match the container with the last launched APP*/
	if (LastLaunchedApp) {
		LastLaunchedApp->m_container_wid = pid;

#ifdef VIRTUAL_WINDOWS
		LastLaunchedApp->m_virtual_wid = get_current_virtwin();
		printf("Recording app in virtual window %d\n", 
		       LastLaunchedApp->m_virtual_wid);
#endif

		LastLaunchedApp = NULL;
	}

	/*GrMapWindow(pid);*/
	GrSetFocus(wid);	/* force fixed focus*/

	/* add client window*/
	add_window(wid, pid, 0, client_wndproc);

#ifdef VIRTUAL_WINDOWS
	/* Redraw the virtual window switcher to handle the missing window */
	/* virtswitch_redraw(); */
#endif

	return 0;
}

/*
 * We've just received an event notifying us that a client window has been
 * unmapped, so we need to destroy all of the decorations.
 */
static void
client_destroy(win *window)
{
	win 		*pwin;
	GR_WINDOW_ID 	pid;

	Dprintf("Client window %d has been destroyed\n", window->wid);

	if(!(pwin = find_window(window->pid))) {
		fprintf(stderr, "Couldn't find parent of destroyed window "
				"%d\n", window->wid);
		return;
	}

	/* Do it this way around so we don't handle events after destroying */
	pid = pwin->wid;
	remove_window_and_children(pwin);

	Dprintf("Destroying container %d\n", pid);
	GrDestroyWindow(pid);

#ifdef VIRTUAL_WINDOWS
	/* Redraw the virtual window switcher to handle the missing window */
	/* virtswitch_redraw(); */
#endif

}

static void
client_update(win *window, GR_EVENT_UPDATE *ep)
{
	switch(ep->utype) {
	case GR_UPDATE_DESTROY:
		client_destroy(window);
		break;
	}
}

static void
client_wndproc(win *window, GR_EVENT *ep)
{
	switch(ep->type) {
	case GR_EVENT_TYPE_CHLD_UPDATE:
		client_update(window, &ep->update);
		break;
	}
}
