/* EXEC.C

   This is the code that handles the launching of new 
   applications from the window manager 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "apps.h"
#include "nanowm.h"

#define ASSOC_FILE "config.magic"

NXLISTHEAD active_list;

static PROCESS* find_process(int pid)
{
  PNXLIST	p = active_list.head;
  
  for (; p; p=p->next) 
    {
      PROCESS *process = nxItemAddr(p, PROCESS, next);
      
      if (process->pid == pid)
	return(process);
    }
  
  return NULL;
}

void kill_runnning_processes()
{
  PNXLIST	p = active_list.head;
  
  while(p)
    {
      PNXLIST n = p->next;
      PROCESS *process = nxItemAddr(p, PROCESS, next);
      
      kill(process->pid, SIGTERM);
      p = n;
    }
}

void application_handler(int id)
{
  int 	status;
  int 	ret;
  
  /* if execlp() fails, we'll end up here before any new mapwindow...*/
  LastLaunchedApp = NULL;

  while (1) 
    {
      ret = waitpid(-1, &status, WNOHANG);
      if (ret == -1 || ret == 0) break;
      else 
	{
	  PROCESS *proc = find_process(ret);

	  if (!proc)
	    return;

	  if (proc->app) 
	    {
	      proc->app->m_process_id = 0;
	      proc->app->m_container_wid = 0;
	    }	
	  
	  /* Remove it from the active list */
	  nxListRemove(&active_list, &proc->next);
	}
    }
}

int launch_application(APP *app)
{ 
  PROCESS *proc;

  /* Check to see if the application is currently running */
  
  if (app->m_process_id && app->m_container_wid)
    {
      win *window;

      /* It was previously running */
      /* Then show it */

      g_last_window = app->m_icon_wid;

      GrMapWindow(app->m_container_wid);
      GrRaiseWindow(app->m_container_wid);

      window = find_window(app->m_container_wid);
            
      if (window) GrSetFocus(window->clientid);
      return(0);
    }

  /* Ok, it wasn't running, so create a new process */
  /* item and start the proceedings                 */
  
  proc = nxItemNew(PROCESS);
  
  if (!(proc->pid = fork())) 
    {
      char path[256];
      char *p;

      /* Change to the working directory before executing */

      p = strrchr(app->m_exec, '/');
      strncpy(path, app->m_exec, (int) (p - app->m_exec));
      path[(int) (p - app->m_exec)] = 0;

      if (chdir(path)) printf("Unable to switch to the working directory %s\n", path);

      /* Start the desired app */
      execvp(app->m_exec, app->m_argv);
      exit(0);
    } 

  if (proc->pid == -1)
    {
      free(proc);
      return(-1);
    }

  proc->app = app;
  app->m_process_id = proc->pid;

  g_last_window = app->m_icon_wid;
  LastLaunchedApp = app;

  nxListAdd(&active_list, &proc->next);
  return(0);
}

void hide_application(APP *app)
{
  GrUnmapWindow(app->m_container_wid);
}

void show_application(APP *app)
{
  GrMapWindow(app->m_container_wid);
  GrRaiseWindow(app->m_container_wid);
}

int is_app_running(APP *app)
{
  return (app->m_process_id && app->m_container_wid);
}


/* File association stuff that may be used in the future? */

#ifdef NOTUSED

#define FILE_ASSOC_COUNT 3

typedef struct 
{
  int active;
  int age;
  char application[512];
  char args[256];
} file_assoc_cache_t;
 
file_assoc_cache_t file_assoc_cache[FILE_ASSOC_COUNT];

static int get_new_cache_entry()
{
  int i = 0;
  int use_slot = -1;
  int oldest_slot = -1;
  int oldest_age = 0;

  /* Determine a cache location for this entry. */
  /* Look for an empty entry and age the other entries at the same time */

  for(i = 0; i < FILE_ASSOC_COUNT; i++)
    {
      if (!file_assoc_cache[i].active)
	{
	  if (use_slot == -1) use_slot = i;
	}
      else
	{
	  if (++file_assoc_cache[i].age > oldest_age)
	    {
	      oldest_age = file_assoc_cache[i].age;
	      oldest_slot = i;
	    }
	}
    }

  if (use_slot == -1)
    use_slot = oldest_slot;

  return(use_slot);
}


/* get_file_association()
   Search the associations file to see if 
   an application has been specified for this file 
*/

static void assoc_callback(char *key, char *value, int data)
{
  if (!strcmp(key, "app"))
    {
      strcpy(file_assoc_cache[data].application, value);
      return;
    }
  
  if (!strcmp(key, "args"))
    {
      strcpy(file_assoc_cache[data].args, value);
      return;
    }
}

/* Stuff for the future? */

int get_file_association(char *filename, char *type, app_info_t *ainfo)
{
  char buf[4096];
  int slot;

  char *afile;

  /* Search for the global configuration file and store the location */	
  nxLoadConfigFile(ASSOC_FILE, NX_CONFIG_ASSOC);

  afile = nxGetConfigFile(NX_CONFIG_ASSOC);
  if (!afile){
    printf("Unable to read the file %s\n");
    return(-1);
  }

  slot = get_new_cache_entry();

  bzero(&file_assoc_cache[slot], sizeof(file_assoc_cache_t));

  file_assoc_cache[slot].active = 1;

  if (IniGetString(type, NULL, "", buf, sizeof(buf), afile))
    {
      IniEnumKeyValues(buf, assoc_callback, slot);

      /* Now, construct the app info */
      strcpy(ainfo->path, file_assoc_cache[slot].application);

      str_replace_variable("$file", filename, file_assoc_cache[slot].args,
			   ainfo->args, sizeof(ainfo->args));\

      return(0);
    }
  
  return(-1);
}
  
#endif



