/*
 * Window Manager/Screentop for Nano-X
 *
 * Copyright (c) 2000 Century Software <embedded.centurysoftware.com>
 *
 * Written by Jason Kingan and Greg Haerr
 */
#include <stdio.h>
#include <stdlib.h>
#include "nano-X.h"
#include "apps.h"
#include "nanowm.h"
#include "settings.h"
#include "catagories.h"

#ifdef SYSTEM_MENUS
#include "sys_menu.h"
#endif

/* External functions called from within */


static GR_SCREEN_INFO	si;

static void root_wndproc(win * window, GR_EVENT * ep);
static void root_prepare_background(void);
static void root_redraw();

#ifdef SYSTEM_MENUS

static void root_reset(void *foo)
{
   wm_load_config_settings();
   root_prepare_background();
   root_redraw();
}

static wm_menu_t *root_sys_menu = 0;
static wm_menu_t *apps_sys_menu = 0;

void close_nxscrtop(int);

static void shutdown_cb(void *foo)
{
  close_nxscrtop(0);
}

#endif
 
void
root_create(void)
{
  int i;
  GR_WM_PROPERTIES props;

    /* add root window*/
  add_window(GR_ROOT_WINDOW_ID, GR_ROOT_WINDOW_ID, 0, root_wndproc);

  /*
   * update and child update events must be set to hook
   * top level window mappings
   */
  
  GrSelectEvents(GR_ROOT_WINDOW_ID, GR_EVENT_MASK_CHLD_UPDATE |
		 GR_EVENT_MASK_EXPOSURE | GR_EVENT_MASK_BUTTON_DOWN);

  /* Load config settings (color, etc) */
  wm_load_config_settings();
 
  /* Cache the background for speed */
  root_prepare_background();

  /* create all the gadget windows*/

  date_create();
  home_create();
  backlight_create();
  keyboard_create();
  scribble_create();
  battery_create();
  
  /* Set the current catagory to zero (general) */
  /* And force a redraw */

  set_current_catagory(0);

#ifdef VIRTUAL_WINDOWS
  cat_switch_create();
#endif

  props.flags = GR_WM_FLAGS_BACKGROUND;
  props.background = wm_get_color(WM_BGCOLOR);
  GrSetWMProperties(GR_ROOT_WINDOW_ID, &props);

  /* Now redraw ourselves */
  root_redraw();

#ifdef SYSTEM_MENUS

  /* Create the root system menu */
  
  root_sys_menu = create_system_menu(6);
  
  if (create_apps_menu(&apps_sys_menu))
    {
      ADD_SYS_MENU_CMENU(&root_sys_menu[0], "Quick Run...", apps_sys_menu);
      ADD_SYS_MENU_SEP(&root_sys_menu[1]);
      i = 2;
    }
  else
    {
      i = 0;
    }

  ADD_SYS_MENU_REGULAR(&root_sys_menu[i++], "Reset ScreenTop", root_reset, 0, 0);
  ADD_SYS_MENU_REGULAR(&root_sys_menu[i++], "About...", 0, 0, 0);
  ADD_SYS_MENU_REGULAR(&root_sys_menu[i++], "Shutdown", shutdown_cb, 0, 0);
  ADD_SYS_MENU_END(&root_sys_menu[i++]);
#endif

}

/* The following are special caching     */
/* routines to make the background image */
/* rendering run faster                  */

static GR_IMAGE_ID lastBGImage = 0;
static GR_WINDOW_ID lastBGPixmap = 0;
static int lastBGWidth = 0;
static int lastBGHeight = 0;

static void root_prepare_background(void)
{
  GR_GC_ID gc = GrNewGC();
  GR_IMAGE_INFO iinfo;
  GR_IMAGE_ID iid;
  int style = 0;
  int wpactive = wm_get_wallpaper(&iid, &style);
  int xpos, ypos, wi, hi;
  if (!si.rows)
    GrGetScreenInfo(&si);
  
  if (!wpactive || lastBGImage == iid)
    return;

  GrGetImageInfo(iid, &iinfo);
  
  /* Remove the last cached BG */

  if (lastBGPixmap)
    GrDestroyWindow(lastBGPixmap);

  /* Figure out the size of the cached pixmap */

  if (style == WALLPAPER_TILED || style == WALLPAPER_FULLSCREEN)
    {
      lastBGPixmap = GrNewPixmap(si.ws_width, si.ws_height, NULL);
      lastBGWidth = si.ws_width;
      lastBGHeight = si.ws_height;
    }
  else /* CENTERED */
    {
      lastBGPixmap = GrNewPixmap(iinfo.width, iinfo.height, NULL);
      lastBGWidth = iinfo.width;
      lastBGHeight = iinfo.height;
    }
  
  GrSetGCForeground(gc, wm_get_color(WM_BGCOLOR));
  GrFillRect(lastBGPixmap, gc, 0, 0, lastBGWidth, lastBGHeight);

  lastBGImage = iid;
  
  
  /* Now draw into the pixmap.  If it is fullscreen or centered,
     thats trival.  Tiled is a little more bad */
  
  switch (style)
    {
    case WALLPAPER_TILED:
      
      for(ypos = 0; ypos < si.ws_height; ypos += iinfo.width)
	for(xpos = 0; xpos < si.ws_width; xpos += iinfo.height)
	  {
	    if (si.ws_width - xpos < iinfo.width)
	      wi = si.ws_width - xpos;
	    else
	      wi = iinfo.width;
	    
	    if (si.ws_height - ypos < iinfo.height)
	      hi = si.ws_height - ypos;
	    else
	      hi = iinfo.height;

	    GrDrawImageToFit(lastBGPixmap, gc, xpos, ypos, 
			     wi, hi, iid);
	  }

      break;
      
    case WALLPAPER_FULLSCREEN:
    case WALLPAPER_CENTERED:
    default:
      GrDrawImageToFit(lastBGPixmap, gc, 0, 0, 
		       -1, -1, iid);
      break;
    }

  GrDestroyGC(gc);
}
  
static void root_draw_background(GR_WINDOW_ID wid, GR_GC_ID gc, 
				 GR_RECT *rc)
{
  GR_IMAGE_ID iid;
  int style = 0;
  int wpactive = wm_get_wallpaper(&iid, &style);

  if (!si.rows)
    GrGetScreenInfo(&si);

  /* First, draw the background (if it is visible) */

  if (!wpactive || style == WALLPAPER_BOTTOM || style == WALLPAPER_CENTERED ||
      !lastBGPixmap)
    {
      GrSetGCForeground(gc, wm_get_color(WM_BGCOLOR));
      GrFillRect(wid, gc, 0, 0, si.ws_width, si.ws_height);
    }
  
  /* Now draw the wall paper */
  if (wpactive && lastBGPixmap)
    {
      int xpos, ypos, wi, hi;
      
      switch (style)
	{
	case WALLPAPER_TILED:
	case WALLPAPER_FULLSCREEN:
	  GrCopyArea(wid, gc, rc->x, rc->y, rc->width, rc->height, 
		     lastBGPixmap, rc->x, rc->y, MWROP_SRCCOPY);
	  
	  break;

	case WALLPAPER_BOTTOM:
	  xpos = (si.ws_width - lastBGWidth) / 2;
	  ypos = (si.ws_height - lastBGHeight);
	  wi = lastBGWidth;
	  hi = lastBGHeight;
	  
	  GrCopyArea(wid, gc, xpos, ypos, wi, hi, 
		     lastBGPixmap, 0, 0, MWROP_SRCCOPY);
	  break;

	case WALLPAPER_CENTERED:
	default:
	  xpos = (si.ws_width - lastBGWidth) / 2;
	  ypos = (si.ws_height - lastBGHeight) / 2;
	  wi = lastBGWidth;
	  hi = lastBGHeight;
	  
	  GrCopyArea(wid, gc, xpos, ypos, wi, hi, 
		     lastBGPixmap, 0, 0, MWROP_SRCCOPY);
	  break;
	}
    }
}

static void
root_exposure(win *window, GR_EVENT_EXPOSURE *ep)
{
	GR_GC_ID 	gc = GrNewGC();
	GR_FONT_ID	fnt = GrCreateFont(GR_FONT_GUI_VAR, 0, NULL);
	GR_REGION_ID	r;
	GR_RECT		rc;

	
	if (!si.rows)
		GrGetScreenInfo(&si);
	/* 
	 * Create clip region based on exposed rectangle.
	 * This keeps flicker way down.
	 */
	r = GrNewRegion();
	rc.x = ep->x;
	rc.y = ep->y;
	rc.width = ep->width;
	rc.height = ep->height;
	GrUnionRectWithRegion(r, &rc);
	GrSetGCRegion(gc, r);

	GrSetGCFont(gc, fnt);

	/* draw taskbar */
	GrSetGCForeground(gc, COLOR_TASKBAR);
	GrFillRect(ep->wid, gc, 0, si.ws_height, si.ws_width,
		si.vs_height - si.ws_height);

	/* draw taskbar top highlight */
	GrSetGCForeground(gc, COLOR_TASKBARHILIGHT);
	GrLine(ep->wid, gc, 0, si.ws_height, si.ws_width - 1, si.ws_height);

	/* draw background (but only the rectangle) */
	root_draw_background(ep->wid, gc, &rc);

	draw_current_iconlist();
	draw_catswitch(&rc);

#ifdef NOTUSED
	/* draw icons*/
	GrSetGCForeground(gc, wm_get_color(WM_ICONTEXT));
	GrSetGCBackground(gc, wm_get_color(WM_ICONCOLOR));

	
	p = apphead.head;

	for (; p; p=p->next) {
		app = nxItemAddr(p, APP, link);

		if (app->m_icon_iid) {
			GrGetWindowInfo(app->m_icon_wid, &info);
			GrDrawImageToFit(GR_ROOT_WINDOW_ID, gc,
				info.x, info.y, -1, -1, app->m_icon_iid);

			/* center text under icon*/
			GrGetGCTextSize(gc, app->m_icontitle, -1,
				GR_TFASCII|GR_TFTOP, &width, &height,&baseline);
			x = (ICONWIDTH - width)/2;
			if (x < 0)
				x /= 2;	/* try to center under icon*/
			x += info.x;
			GrText(GR_ROOT_WINDOW_ID, gc, x, info.y+ICONHEIGHT,
				app->m_icontitle, -1, GR_TFASCII|GR_TFTOP);
					
		}
	}

#endif
	GrDestroyFont(fnt);
	GrDestroyGC(gc);
	GrDestroyRegion(r);
}

static void root_redraw()
{
  GR_EVENT_EXPOSURE 	expose;
  GR_WINDOW_INFO	info;
  
  /* dummy full area expose event*/
  GrGetWindowInfo(GR_ROOT_WINDOW_ID, &info);
  expose.type = GR_EVENT_TYPE_EXPOSURE;
  expose.wid = GR_ROOT_WINDOW_ID;
  expose.x = 0;
  expose.y = 0;
  expose.width = info.width;
  expose.height = info.height;
  root_exposure(0, &expose);
}

inline void redraw_root_window()
{
  root_redraw();
}

static void
root_wndproc(win * window, GR_EVENT * ep)
{
	switch (ep->type) {
	case GR_EVENT_TYPE_EXPOSURE:
		root_exposure(window, &ep->exposure);
		break;

#ifdef SYSTEM_MENUS
	case GR_EVENT_TYPE_BUTTON_DOWN:
	  
	  if (ep->button.buttons & GR_BUTTON_R)
	    {
	      open_system_menu(GR_ROOT_WINDOW_ID,
			       ep->button.x, ep->button.y, MENU_DIR_BEST,
			       root_sys_menu);
	    }

	  break;
#endif
	}
}

void root_free_memory()
{

#ifdef SYSTEM_MENUS
  destroy_system_menu(root_sys_menu);

#endif

}
