/*******************************************************************************\
**
**	Program:	ads_blctrl.c
**	Desc:		Test program to demonstrate the control of the ADS GraphicsClient
**				Plus backlight (either through the contrast, or on/off)
**	History:	11/29/2000	Jeff Webb <jeffw@censoft.com>
**
\*******************************************************************************/

/* Feature test macros */


/* System header files */
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

/* Local header files */
#include "smartio.h"

/* Typedef, macro, enum/struct/union definitions */

/* Global scope variables */

/* File scope variables */

/* Ansi C Function prototyping */


/*******************************************************************************\
**
**	Function:	int Ads_Bl_Control()
**	Desc:		Turns the ADS Unit's backlight control either on or off depending
**				on the value of cmd
**	Accepts:	char cmd = The backlight action to process. 0 = off, 1 = on
**	Returns:	int; 0 on success, -1 on error (the return value from ioctl())
**
\*******************************************************************************/
int
Ads_Bl_Control(char cmd)
{
	int					blfd,									/* Backlight file descriptor */
						rc;										/* Return code */

	/* Simplify value to only 0 = off, 1 = on */
	cmd = (cmd != 0);
	
	if ( (rc = blfd = open(DEV_BACKLITE, O_RDWR)) > -1)
	{
		rc = ioctl(blfd, SMARTIO_BL_CONTROL, &cmd);
		close(blfd);
	} /* end of if */

	return (rc);
} /* end of Ads_Bl_Control */

/*******************************************************************************\
**
**	Function:	int Ads_Bl_Contrast()
**	Desc:		Sets the intensity level of the ADS GraphicsClient+ backlight
**	Accepts:	int mode = Controls interpretation of value:
**					0 = Treat value as a percent
**					1 = Treat value as actual value
**				unsigned char value = contrast value (0-brightest, 255-darkest)
**	Returns:	int; 0 on success, -1 on error (the return value of ioctl())
**
\*******************************************************************************/
int
Ads_Bl_Contrast(int mode, unsigned char value)
{
	int					blfd,									/* Backlight file descriptor */
						rc;										/* Return code */

	if (!mode)
		value = ( (int)(255 * ((100 - value) / 100.0)) & 0xff);
	else
		value &= 0xff;

	if ( (rc = blfd = open(DEV_BACKLITE, O_RDWR)) > -1)
	{
		rc = ioctl(blfd, SMARTIO_BL_CONTRAST, &value);
		close (blfd);
	} /* end of if */

	return (rc);
} /* end of Ads_Bl_Contrast */

#ifdef LIBDEBUG
/* main() */
int
main(int argc, char **argv)
{
	unsigned char	blctrst = 0,						/* Backlight contrast level */	
					bloflg = 1,							/* Backlight on flag (0 = turn off, 1 = turn on) */
					ctrstflg = 0;						/* Contrast flag */
	int				errflg = 0,							/* Error flag */
					gopt,								/* Get option */
					tmpInt;								/* Temporary int storage */

	while ( (gopt = getopt(argc, argv, "c:fnr:")) != -1) switch (gopt)
	{
		case 'c':										/* Set the contrast level in percent */
			if (optarg && sscanf(optarg, "%d", &tmpInt) < 1)
				tmpInt = 0;
			blctrst = tmpInt % 100;
			ctrstflg = 1;
			break;
		case 'f':			/* Turn the light off */
			bloflg = 0;
			break;
		case 'n':
			bloflg = 1;
			break;
		case 'r':										/* Raw contrast level */
			if (optarg && sscanf(optarg, "%d", &tmpInt) < 1)
				tmpInt = 0;
			blctrst = tmpInt & 0xff;
			ctrstflg = 2;
			break;
	} /* end of while-switch */

	if (bloflg == 1)
	{
		/* Turn the backlight on and set the contrast */
		if (Ads_Bl_Control(bloflg) == -1)
		{
			printf("ioctl() failed, error=%d.\n", errno);
		} /* end of if */
	} /* end of if */
	if (ctrstflg)
	{
		if (Ads_Bl_Contrast(ctrstflg == 1 ? 0 : 1, blctrst) == -1)
		{
			printf("Error! setting backlight contrast to %d, error=%s.\n", blctrst, strerror(errno));
		} /* end of if */
	} /* end of if */
	if (bloflg == 0)
	{
		if (Ads_Bl_Control(bloflg) == -1)
		{
			printf("Error! Unable to turn backlight off, error=%s\n", strerror(errno));
		} /* end of if */
	} /* end of else */

} /* end of main() */
#endif
